package com.sawerli.plugin;

import android.content.Intent;
import android.content.res.ColorStateList;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.ImageButton;

import com.getcapacitor.PluginRequestCodes;
import com.google.android.gms.vision.CameraSource;
import com.google.android.gms.vision.Tracker;
import com.google.android.gms.vision.face.Face;
import com.google.android.gms.vision.face.FaceDetector;
import com.google.android.gms.vision.face.LargestFaceFocusingProcessor;
import com.sawerli.plugin.capacitorcustomcamera.R;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;


public class CustomCameraActivity extends AppCompatActivity {
    private SurfaceView surfaceView;
    private CameraSource cameraSource;
    private SurfaceHolder surfaceHolder;
    private FaceDetector detector;

    private ImageButton galleryBtn;
    private ImageButton captureBtn;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);


        setContentView(R.layout.custom_cam_view);
        surfaceView = findViewById(R.id.surfaceView);
        detector = new FaceDetector.Builder(this).build();
        captureBtn = (ImageButton) this.findViewById(R.id.btn_capture);
        captureBtn.setSelected(false);
        galleryBtn = (ImageButton) this.findViewById(R.id.btn_gallery);

        if (!detector.isOperational()) {
            Log.w("MainActivity", "Detector Dependencies are not yet available");
        } else {
            Log.w("MainActivity", "Detector Dependencies are available");
            if (surfaceView != null) {
                //setViewVisibility(R.id.tv_capture);
                //setViewVisibility(R.id.surfaceView);
                captureBtn.setVisibility(View.VISIBLE);
                galleryBtn.setVisibility(View.VISIBLE);
                findViewById(R.id.surfaceView).setVisibility(View.VISIBLE);
                setupSurfaceHolder();
            }
        }

        captureBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                clickImage();
                captureBtn.setSelected(true);
            }
        });

        galleryBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                openGallery();
                //galleryBtn.setColorFilter(Color.argb(255, 94, 191, 190));
                galleryBtn.setColorFilter(getResources().getColor(R.color.secondary));
            }
        });
    }

    private void setupSurfaceHolder() {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        int width = displayMetrics.widthPixels;
        int height = displayMetrics.heightPixels;


        cameraSource = new CameraSource.Builder(this, detector)
                .setRequestedPreviewSize(1280, 720) // Hardcoded, for now.
                .setFacing(CameraSource.CAMERA_FACING_BACK)
                .setRequestedFps(60.0f)
                .setAutoFocusEnabled(true)
                .build();

        surfaceView.setLayoutParams(new FrameLayout.LayoutParams(width, width/9*16));

        surfaceHolder = surfaceView.getHolder();
        surfaceHolder.addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                try {
                    cameraSource.start(surfaceHolder);
                    detector.setProcessor(new LargestFaceFocusingProcessor(detector,
                            new Tracker<Face>()));
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                cameraSource.stop();
            }
        });
    }

    private void clickImage() {
        if (cameraSource != null) {
            cameraSource.takePicture(/*shutterCallback*/null, new CameraSource.PictureCallback() {
                @Override
                public void onPictureTaken(byte[] bytes) {
                    Bitmap bitmap = BitmapFactory.decodeByteArray(bytes, 0, bytes.length);
                    Matrix matrix = new Matrix();
                    matrix.setRotate(90);
                    bitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);

                    /* ((ImageView) findViewById(R.id.iv_picture)).setImageBitmap(bitmap);
                    //setViewVisibility(R.id.iv_picture);
                    findViewById(R.id.iv_picture).setVisibility(View.VISIBLE);
                    findViewById(R.id.surfaceView).setVisibility(View.GONE);
                    findViewById(R.id.tv_capture).setVisibility(View.GONE);
                    findViewById(R.id.tv_gallery).setVisibility(View.GONE); */

                    try {
                        ByteArrayOutputStream bitmapOutputStream = new ByteArrayOutputStream();
                        bitmapOutputStream.write(bytes, 0, bytes.length);
                        ByteArrayInputStream bis = new ByteArrayInputStream(bitmapOutputStream.toByteArray());
                        Uri contentUri = Uri.parse("cam-capture"); //Uri.fromFile(f);

                        if (bitmap != null) {

                            // Compress the final image and prepare for output to client
                            Uri uri = saveTemporaryImage(bitmap, contentUri, bis);
                            Intent data = new Intent();
                            data.putExtra("uri", uri.toString());
                            setResult(RESULT_OK, data);
                            finish();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
        }
    }



    /**
     * Save the modified image we've created to a temporary location, so we can
     * return a URI to it later
     * @param bitmap
     * @param contentUri
     * @param is
     * @return
     * @throws IOException
     */
    private Uri saveTemporaryImage(Bitmap bitmap, Uri contentUri, InputStream is) throws IOException {
        String filename = contentUri.getLastPathSegment();
        if (!filename.contains(".jpg") && !filename.contains(".jpeg")) {
            filename += "." + (new java.util.Date()).getTime() + ".jpeg";
        }

        File cacheDir = this.getCacheDir();
        File outFile = new File(cacheDir, filename);
        FileOutputStream fos = new FileOutputStream(outFile);

        try {
            if(!outFile.exists()){
                outFile.mkdirs();
            }
            bitmap.compress(Bitmap.CompressFormat.JPEG, 90, fos);
            fos.close();
        } catch (Exception e) {
            Log.e("error", "Saving received message failed with", e);
        }
        return Uri.fromFile(outFile);

    }


    static final int REQUEST_IMAGE_PICK = PluginRequestCodes.CAMERA_IMAGE_PICK;
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        // Check which request we're responding to

        if (data != null) {
            Uri uri = data.getData();

            if (requestCode == REQUEST_IMAGE_PICK) {
                //processPickedImage(savedCall, data);
                Intent mIntent = new Intent();
                mIntent.putExtra("uri_gallery", uri.toString());
                setResult(RESULT_OK, data);
                finish();
            }
        }

    }


    private void openGallery() {
        final int REQUEST_IMAGE_PICK = PluginRequestCodes.CAMERA_IMAGE_PICK;
        Intent intent = new Intent(Intent.ACTION_PICK);
        intent.setType("image/*");
        startActivityForResult(intent, REQUEST_IMAGE_PICK);
    }
    public void onBackClicked(View view) {
        finish();
    }


}
